/* Copyright (c) 2006-2008 MetaCarta, Inc., published under the Clear BSD
 * license.  See http://svn.openlayers.org/trunk/openlayers/license.txt for the
 * full text of the license. */


/**
 * @requires OpenLayers/Control.js
 */

/**
 * Class: OpenLayers.Control.ArgParser
 * The ArgParser control adds location bar querystring parsing functionality 
 * to an OpenLayers Map.
 * When added to a Map control, on a page load/refresh, the Map will 
 * automatically take the href string and parse it for lon, lat, zoom, and 
 * layers information. 
 *
 * Inherits from:
 *  - <OpenLayers.Control>
 */
MappingKit.Control.ArgParser = OpenLayers.Class(OpenLayers.Control.ArgParser, {

    /**
     * Parameter: args
     * {String}
     */
    args: null,
    
    /**
     * Parameter: cookies
     * {boolean}
     */
    cookies: false,
    
    /**
     * Constructor: OpenLayers.Control.ArgParser
     *
     * Parameters:
     * options - {Object}
     */
    initialize: function(options) {
        OpenLayers.Control.ArgParser.prototype.initialize.apply(this, arguments);
    },

    /**
     * Method: setMap
     * Set the map property for the control. 
     * 
     * Parameters:
     * map - {<OpenLayers.Map>} 
     */
    setMap: function(map) {
        // call Control, not the ArgParser parent class
        OpenLayers.Control.prototype.setMap.apply(this, arguments);

        // check if cookies are used for this map
        if (this.map.options) {
          this.cookies = this.map.options.useCookies;
        }
        
        //make sure we dont already have an arg parser attached
        for(var i=0, len=this.map.controls.length; i<len; i++) {
            var control = this.map.controls[i];
            if ( (control != this) &&
                 (control.CLASS_NAME == "OpenLayers.Control.ArgParser")) {
                
                // If a second argparser is added to the map, then we 
                // override the displayProjection to be the one added to the
                // map. 
                if (control.displayProjection != this.displayProjection) {
                    this.displayProjection = control.displayProjection;
                }
                
                break;
            }
        }
        if (i == this.map.controls.length) {

            var args = {};
            if (this.args) {
              // get parameters from args string
              args = OpenLayers.Util.getParameters('?' + this.args);
            }
            // overwrite with parameters from url
            OpenLayers.Util.extend(args, OpenLayers.Util.getParameters());
            
            if (this.cookies && document.cookie) {
              // overwrite with parameters from cookie
              var cookies = document.cookie.split(';');
              for (var i = 0, count = cookies.length; i < count; i++) {
                var cookie = cookies[i].trim();
                var len = cookie.indexOf('=');
                var key = cookie.substr(0, len);
                if (key == MappingKit.getMapCookieKey(this.map)) {
                  var value = decodeURIComponent(cookie.substr(len + 1));
                  var cookieParams = OpenLayers.Util.getParameters('?' + value);
                  if (cookieParams.search == window.location.search) {
                    // only if for this same page (including search part)
                    OpenLayers.Util.extend(args, cookieParams);
                  }
                  break;
                }
              }
            }
            
            // Be careful to set layer first, to not trigger unnecessary layer loads
            if (args.layers) {
                this.layers = args.layers;
    
                // when we add a new layer, set its visibility 
                this.map.events.register('addlayer', this, 
                                         this.configureLayers);
                this.configureLayers();
            }
            if (args.lat && args.lon) {
                this.center = new OpenLayers.LonLat(parseFloat(args.lon),
                                                    parseFloat(args.lat));
                if (args.zoom) {
                    this.zoom = parseInt(args.zoom);
                }
    
                // when we add a new baselayer to see when we can set the center
                this.map.events.register('changebaselayer', this, 
                                         this.setCenter);
                this.setCenter();
            }
        }
    },
    
    CLASS_NAME: "MappingKit.Control.ArgParser"
});
