/* Copyright (c) 2006-2008 MetaCarta, Inc., published under the Clear BSD
 * license.  See http://svn.openlayers.org/trunk/openlayers/license.txt for the
 * full text of the license. */

/**
 * @requires OpenLayers/Format/XML.js
 */

/**
 * Class: OpenLayers.Format.OWSContext
 * Read and write OWS Context documents.
 *
 * Inherits from:
 *  - <OpenLayers.Format.XML>
 */
OpenLayers.Format.OWSContext = OpenLayers.Class({
    
    /**
     * APIProperty: defaultVersion
     * {String} Version number to assume if none found.  Default is "0.3.1".
     */
    defaultVersion: "0.3.1",
    
    /**
     * APIProperty: version
     * {String} Specify a version string if one is known.
     */
    version: null,

    /**
     * Property: layerOptions
     * {Object} Default options for layers created by the parser.
     *     Per layer type (identified by CLASS_NAME) an object.
     */
    layerOptions: {}, 

    /**
     * Property: layerParams
     * {Object} Default parameters for layers created by the parser. This
     *     can be used to override DEFAULT_PARAMS.
     *     Per layer type (identified by CLASS_NAME) an object.
     */
    layerParams: {},

    /**
     * APIProperty: groupByService 
     * {Boolean} Set true to group layers from the same WMS service into
     *     one layer. Layers are considered to be from the same WMS when
     *     layer url's match. Defaults to false.
     */
    groupByService: false, 
    
    /**
     * Property: parser
     * {Object} Instance of the versioned parser.  Cached for multiple read and
     *     write calls of the same version.
     */
    parser: null,

    /**
     * Constructor: OpenLayers.Format.OWSContext
     * Create a new parser for OWSContext docs.
     *
     * Parameters:
     * options - {Object} An optional object whose properties will be set on
     *     this instance.
     */
    initialize: function(options) {
        OpenLayers.Util.extend(this, options);
        this.options = options;
    },

    /**
     * APIMethod: read
     * Read OWSContext data from a string, and return an object with map properties
     *     and a list of layers. 
     * 
     * Parameters: 
     * data - {String} or {DOMElement} data to read/parse.
     * options - {Object} The options object must contain a map property.  If
     *     the map property is a string, it must be the id of a dom element
     *     where the new map will be placed.  If the map property is an
     *     <OpenLayers.Map>, the layers from the context document will be added
     *     to the map.
     *
     * Returns:
     * {<OpenLayers.Map>} A map based on the context.
     */
    read: function(data, options) {
        if(typeof data == "string") {
            data = OpenLayers.Format.XML.prototype.read.apply(this, [data]);
        }
        var root = data.documentElement;
        var version = this.version;
        if(!version) {
            version = root.getAttribute("version");
            if(!version) {
                version = this.defaultVersion;
            }
        }
        if(!this.parser || this.parser.VERSION != version) {
            var format = OpenLayers.Format.OWSContext[
                "v" + version.replace(/\./g, "_")
            ];
            if(!format) {
                throw "Can't find a OWSContext parser for version " +
                      version;
            }
            this.parser = new format(this.options);
        }
        var context = this.parser.read(data, options);
        var map;
        if(options && options.map) {
            this.context = context;
            if(options.map instanceof OpenLayers.Map) {
                map = this.mergeContextToMap(context, options.map);
            } else {
                map = this.contextToMap(context, options.map);
            }
        } else {
            // not documented as part of the API, provided as a non-API option
            map = context;
        }
        return map;
    },
    
    /**
     * Method: contextToMap
     * Create a map given a context object.
     *
     * Parameters:
     * context - {Object} The context object.
     * id - {String | Element} The dom element or element id that will contain
     *     the map.
     *
     * Returns:
     * {<OpenLayers.Map>} A map based on the context object.
     */
    contextToMap: function(context, id) {
        var map = new OpenLayers.Map(id, {
            allOverlays: true,
            maxExtent: context.maxExtent,
            projection: context.projection
        });
        map.addLayers(context.layers);
        map.setCenter(
            context.bounds.getCenterLonLat(),
            map.getZoomForExtent(context.bounds, true)
        );
        return map;
    },
    
    /**
     * Method: mergeContextToMap
     * Add layers from a context object to a map.
     *
     * Parameters:
     * context - {Object} The context object.
     * map - {<OpenLayers.Map>} The map.
     *
     * Returns:
     * {<OpenLayers.Map>} The same map with layers added.
     */
    mergeContextToMap: function(context, map) {
      if (context.baselayers) {
        var baselayers = context.baselayers.split(',');
        var layers = [];
        for (var i = 0; i < baselayers.length; i++) {
          //var base_options = {isBaseLayer: true, sphericalMercator: true};
          var options = { isBaseLayer: true, sphericalMercator: true };
          switch (baselayers[i]) {
          case 'google':
            if (typeof G_HYBRID_MAP != 'undefined') {
              options.type = G_HYBRID_MAP;
              layers.push(new OpenLayers.Google('Google', options));
            }
            break;
          case 'none':
            layers.push(new OpenLayers.Layer(Drupal.t('None'), options));
            break;
          case 'osm':
            options.type = 'png';
            options.getURL = osm_getTileURL;
            options.displayOutsideMaxExtent = true;
            layers.push(new OpenLayers.Layer.TMS('OpenStreetMap', 'http://tile.openstreetmap.org/', options));
            break;
          case 've':
          case 'bing':
            if (typeof VEMapStyle != 'undefined') {
              options.type = VEMapStyle.Hybrid;
              layers.push(new OpenLayers.Layer.VirtualEarth('Bing', options));
            }
            break;
          }
          
          if (layers.length > 0) {
            // change map options if projection != EPSG:900913 or equivalent
            var options = { sphericalMercator: true, units: 'm' };
            options.projection = new OpenLayers.Projection('EPSG:900913');
            options.displayProjection = new OpenLayers.Projection('EPSG:4326');
            options.maxExtent = new OpenLayers.Bounds(-20037508.34, -20037508.34, 20037508.34, 20037508.34);
            options.maxResolution = 2 * 20037508.34 / 256;
            map.setOptions(options);
            
            // add base layers
            map.addLayers(layers);
          }
        }
      }
      
      if (!map.baseLayer && context.layers.length > 0) {
        // the first layer in this context will be the base layer
        context.layers[0].isBaseLayer = true;
      }
      
      if (context.maxExtent) {
        // extend bounds
        map.maxExtent.extend(context.maxExtent);
      }
      
      // add normal layers
      map.addLayers(context.layers);
      
      // use the context title
      if (!map.title) {
        map.title = context.title;
      }
      return map;
    },

    /**
     * APIMethod: write
     * Write a WMC document given a map.
     *
     * Parameters:
     * obj - {<OpenLayers.Map> | Object} A map or context object.
     * options - {Object} Optional configuration object.
     *
     * Returns:
     * {String} A WMC document string.
     */
    write: function(obj, options) {
        if(obj.CLASS_NAME == "OpenLayers.Map") {
            obj = this.mapToContext(obj);
        }
        var version = (options && options.version) ||
                      this.version || this.defaultVersion;
        if(!this.parser || this.parser.VERSION != version) {
            var format = OpenLayers.Format.OWSContext[
                "v" + version.replace(/\./g, "_")
            ];
            if(!format) {
                throw "Can't find a OWSContext capabilities parser for version " +
                      version;
            }
            this.parser = new format(this.options);
        }
        var wmc = this.parser.write(obj, options);
        return wmc;
    },
    
    /**
     * Method: mapToContext
     * Create a context object given a map.
     *
     * Parameters:
     * map - {<OpenLayers.Map>} The map.
     *
     * Returns:
     * {Object} A context object.
     */
    mapToContext: function(map) {
        var context = {
            bounds: map.getExtent(),
            maxExtent: map.maxExtent,
            projection: map.projection,
            layers: map.layers,
            title: map.title ? map.title : 'OpenLayers OWSContext',
            size: map.getSize()
        };
        return context;
    },

    CLASS_NAME: "OpenLayers.Format.OWSContext" 

});
