/* Copyright (c) 2006-2009 MetaCarta, Inc., published under the Clear BSD
 * license.  See http://svn.openlayers.org/trunk/openlayers/license.txt for the
 * full text of the license. */

/**
 * @requires OpenLayers/Format/OWSContext.js
 * @requires OpenLayers/Format/XML.js
 * @requires OpenLayers/Format/KML.js 
 * @requires OpenLayers/Format/GML/v2.js 
 * @requires OpenLayers/Format/SLD/v1_0_0.js 
 */

/**
 * Class: OpenLayers.Format.OWSContext.v0_3_1
 * Read and write OWSContext version 0.3.1.
 *
 * Inherits from:
 *  - <OpenLayers.Format.XML>
 */
OpenLayers.Format.OWSContext.v0_3_1 = OpenLayers.Class(OpenLayers.Format.XML, {
    
    /**
     * Property: namespaces
     * {Object} Mapping of namespace aliases to namespace URIs.
     */
    namespaces: {
        owc: "http://www.opengis.net/ows-context",
        gml: "http://www.opengis.net/gml",
        kml: "http://www.opengis.net/kml/2.2",
        ogc: "http://www.opengis.net/ogc",
        ol: "http://openlayers.org/context",
        ows: "http://www.opengis.net/ows",
        sld: "http://www.opengis.net/sld",
        xlink: "http://www.w3.org/1999/xlink",
        xsi: "http://www.w3.org/2001/XMLSchema-instance"
    },

    /**
     * Constant: VERSION
     * {String} 0.3.1
     */
    VERSION: "0.3.1", 

    /**
     * Property: schemaLocation
     * {String} Schema location
     */
    schemaLocation: "http://www.opengis.net/ows-context http://www.ogcnetwork.net/schemas/owc/0.3.1/owsContext.xsd",

    /**
     * Property: defaultPrefix
     */
    defaultPrefix: "owc",

    /**
     * APIProperty: extractAttributes
     * {Boolean} Extract attributes from GML.  Default is true.
     */
    extractAttributes: true,
    
    /**
     * APIProperty: xy
     * {Boolean} Order of the GML coordinate true:(x,y) or false:(y,x)
     * Changing is not recommended, a new Format should be instantiated.
     */ 
    xy: true, 

    /**
     * Property: kmlFormat
     * {<OpenLayers.Format.KML>} An internal KML Format since the KML Format
     *     currently does not have readers and writers so we cannot extend
     *     those.
     */
    kmlFormat: new OpenLayers.Format.KML({extractStyles: true}),

    /**
     * Property: gmlFormat
     * {<OpenLayers.Format.GML>} An internal GML Format used for file-based
     *     GML where we do not know the featureType and the featureNS so
     *     we cannot use the new OpenLayers.Format.GML.v2
     */
    gmlFormat: new OpenLayers.Format.GML(),

    /**
     * Property: regExes
     * Compiled regular expressions for manipulating strings.
     */
    regExes: {
        trimSpace: (/^\s*|\s*$/g),
        removeSpace: (/\s*/g),
        splitSpace: (/\s+/),
        trimComma: (/\s*,\s*/g)
    },

    /**
     * Property: featureNS
     * {String} The namespace uri to use for writing InlineGeometry
     */
    featureNS: "http://mapserver.gis.umn.edu/mapserver",

    /**
     * Property: geometryTypes
     * {Object} The mapping for geometry types taken from GML.v2 
     */
    geometryTypes: OpenLayers.Format.GML.v2.prototype.geometryTypes,

    /**
     * Property: featureType
     * {String} The name to use as the feature type when writing out
     *     InlineGeometry
     */
    featureType: 'vector',
              
    /**
     * Property: geometryName
     * {String} The name to use for the geometry attribute when writing out
     *     InlineGeometry
     */
    geometryName: 'geometry',

    /**
     * Constructor: OpenLayers.Format.OWSContext.v0_3_1
     * Instances of this class are not created directly.  Use the
     *     <OpenLayers.Format.OWSContext> constructor instead.
     *
     * Parameters:
     * options - {Object} An optional object whose properties will be set on
     *     this instance.
     */
    initialize: function(options) {
        OpenLayers.Format.XML.prototype.initialize.apply(this, [options]);
    },

    /**
     * Get the options to use in the Layer constructor
     *
     * Parameters:
     * options - {Object} existing options object
     * className - {String} the layer type as a CLASS_NAME string
     *
     * Returns:
     * {Object} the changed options
     */
    getOptions: function(options, className) {
        if (this.layerOptions[className]) {
            OpenLayers.Util.applyDefaults(options,
                this.layerOptions[className]);
        }
        return options;
    },

    /**
     * Method: getLayerFromInfo
     * Create a layer from a layerInfo object.
     *
     * Parameters:
     * layerInfo - {Object} An object representing a layer.
     *
     * Returns:
     * {<OpenLayers.Layer>} The respective OpenLayers.Layer object or null 
     * when no Layer object was created. 
     */
    getLayerFromInfo: function(layerInfo) {
        if (layerInfo.minScaleDenominator) {
            layerInfo.options.maxScale = layerInfo.minScaleDenominator;
        }
        if (layerInfo.maxScaleDenominator) {
            layerInfo.options.minScale = layerInfo.maxScaleDenominator;
        }
        if (layerInfo.nestingPath){
            layerInfo.options.nestingPath = layerInfo.nestingPath; 
        }
        var params, layer;
        // inline GML or KML features
        if (layerInfo.features) {
            layer = new OpenLayers.Layer.Vector(layerInfo.title, 
                this.getOptions(layerInfo.options, 'OpenLayers.Layer.Vector'));
            layer.addFeatures(layerInfo.features);
        } else if (layerInfo.service) {
            switch (layerInfo.service) {
                case "urn:ogc:serviceType:WMS": 
                    params = this.layerParams['OpenLayers.Layer.WMS'] || {};
                    params.LAYERS = layerInfo.name;
                    params.FORMAT = layerInfo.format;
                    params.VERSION = layerInfo.version;
                    layer = new OpenLayers.Layer.WMS(layerInfo.title, 
                        layerInfo.href, params, this.getOptions(
                            layerInfo.options, 'OpenLayers.Layer.WMS'));
                break;
                case "urn:ogc:serviceType:WFS":
                    layerInfo.options.strategies = [new OpenLayers.Strategy.BBOX()];
                    layerInfo.options.protocol = new OpenLayers.Protocol.WFS({
                        url: layerInfo.href,
                        // since we do not know featureNS, let the protocol
                        // determine it automagically using featurePrefix
                        featurePrefix: layerInfo.name.split(":")[0],
                        featureType: layerInfo.name.split(":").pop()
                    });
                    layer = new OpenLayers.Layer.Vector(layerInfo.title, 
                        this.getOptions(layerInfo.options, 
                            'OpenLayers.Layer.Vector'));
                break;
                case "urn:ogc:serviceType:GML":
                    // use a vector layer with a HTTP Protocol and a Fixed strategy
                    layerInfo.options.strategies = [new OpenLayers.Strategy.Fixed()];
                    layerInfo.options.protocol = new OpenLayers.Protocol.HTTP({
                        url: layerInfo.href, format: this.gmlFormat});
                    layer = new OpenLayers.Layer.Vector(layerInfo.title, 
                        this.getOptions(layerInfo.options, 
                            'OpenLayers.Layer.Vector'));
                break;
                case "urn:ogc:serviceType:KML":
                    // use a vector layer with an HTTP Protcol and a Fixed strategy
                    layerInfo.options.strategies = [new OpenLayers.Strategy.Fixed()];
                    layerInfo.options.protocol = new OpenLayers.Protocol.HTTP({
                        url: layerInfo.href, format: this.kmlFormat});
                    layer = new OpenLayers.Layer.Vector(layerInfo.title, 
                        this.getOptions(layerInfo.options, 
                            'OpenLayers.Layer.Vector'));
                break;
                default: 
                    layer = null;
                break;
            }
        }
        return layer;
    },

    /**
     * Parameters:
     * l - {Object} that may have a layerInfo array property.
     * 
     */
    setNestingPath : function(l){
        if(l.layerInfo){
            for (var i = 0, max = l.layerInfo.length; i < max; i++) {
                var layerInfo = l.layerInfo[i];
                var nPath = "";
                var nTitle = ""; 
                if(l.nestingPath){
                    nPath = l.nestingPath;
                }
                //in theory check for (l.title && !l.name) 
                if(l.title && !l.href  && !l.features){
                    nTitle = l.title;
                }
                if (nPath != "") {
                    nPath = nPath + "/" + nTitle;
                } else {
                    nPath = nTitle;
                }
                layerInfo.nestingPath = nPath;
                if(layerInfo.layerInfo){
                    this.setNestingPath(layerInfo);
                }
            }
        }
    },

    /**
     * Method: read
     * Read OWS context data from a string, and return a list of layers. 
     * 
     * Parameters: 
     * data - {String} or {DOMElement} data to read/parse.
     *
     * Returns:
     * {Array} List of named layers.
     */
    read: function(data) {
        if(typeof data == "string") {
            data = OpenLayers.Format.XML.prototype.read.apply(this, [data]);
        }
        if(data && data.nodeType == 9) {
            data = data.documentElement;
        }
        var context = {};
        context.layers = [];
        this.readNode(data, context);

        // we need to do postprocessing since we do not know the layer type
        // at the time we parse the Layer node

        // since an OWSContext can be nested we need to go through this
        // structure recursively      
        this.setNestingPath({layerInfo : context.layerInfo});
        this.processLayer(context.layers, context);
        return context;
    },

    /**
     * Method: findLayer
     * Check if there is already a layer from the same WMS, if so return it.
     *
     * Parameters:
     * layerArray - {Array} layer array to search in
     * url - {String} url the search for
     *
     * Returns:
     * {<OpenLayers.Layer.WMS>} or {Boolean}
     */
    findLayer: function(layerArray, url) {
        for (var i=0, len=layerArray.length; i<len; i++) {
            var layer = layerArray[i];
            if (layer instanceof OpenLayers.Layer.WMS && layer.url === url) {
                return layer;
            }
        }
        return false;
    },

    /**
     * Method: processLayer
     * Recursive function to process a layer and their childLayers
     *
     * Parameters:
     * layer - {Object} layerInfo object
     */
    processLayer: function(layerArray, layer) {
        if (layer.layerInfo) {
            for (var i=0, len = layer.layerInfo.length; i<len; i++) {
                var l = layer.layerInfo[i];
                var layerObj  = this.getLayerFromInfo(l);
                if (layerObj != null) {
                    var lyr = false;
                    if (this.groupByService) {
                        // group layers according to Service url
                        lyr = this.findLayer(layerArray, layerObj.url);
                    }
                    if (lyr !== false) {
                        lyr.params.LAYERS += "," + layerObj.params.LAYERS;
                    } else {
                        layerArray.push(layerObj);
                    }
                }
                if (l.layerInfo) {
                    this.processLayer(layerArray, l);
                }
            }
        }
    },

    /**
     * Method: write
     *
     * Parameters:
     * context - {Object} An object representing the map context.
     * options - {Object} Optional object.
     *
     * Returns:
     * {String} An OWS Context document string.
     */
    write: function(context, options) {
        var name = "OWSContext";
        var root = this.writeNode(name, context);
        this.setAttributeNS(
            root, this.namespaces["xsi"],
            "xsi:schemaLocation", this.schemaLocation
        );

        return OpenLayers.Format.XML.prototype.write.apply(this, [root]);
    }, 

    /**
     * Property: readers
     * Contains public functions, grouped by namespace prefix, that will
     *     be applied when a namespaced node is found matching the function
     *     name.  The function will be applied in the scope of this parser
     *     with two arguments: the node being read and a context object passed
     *     from the parent.
     */
    readers: {
        "kml": {
            "Document": function(node, obj) {
                obj.features = this.kmlFormat.read(node);
            }
        },
        "owc": { 
            "OWSContext": function(node, obj) {
                this.readChildNodes(node, obj);
            }, 
            "General": function(node, obj) {
                this.readChildNodes(node, obj);
            },
            "ResourceList": function(node, obj) {
                this.readChildNodes(node, obj);
            },
            "Layer": function(node, obj) {
                var layerInfo = {
                    options: {
                        visibility: (node.getAttribute("hidden") != "1"),
                        queryable: (node.getAttribute("queryable") == "1")
                    }
                    
                };
                if(node.getAttribute("name") !== null){
                   layerInfo.name = node.getAttribute("name"); 
                }
                if (!obj.layerInfo) {
                    obj.layerInfo = [];
                }
                obj.layerInfo.push(layerInfo);
                this.readChildNodes(node, layerInfo);
            },
            "InlineGeometry": function(node, obj) {
                obj.features = [];
                var elements = this.getElementsByTagNameNS(node, 
                    this.namespaces.gml, "featureMember");
                var el;
                if (elements.length >= 1) {
                    el = elements[0];
                }
                if (el && el.firstChild) {
                    var featurenode = (el.firstChild.nextSibling) ? 
                        el.firstChild.nextSibling : el.firstChild;
                    this.setNamespace("feature", featurenode.namespaceURI);
                    this.featureType = featurenode.localName || 
                        featurenode.nodeName.split(":").pop();
                    this.readChildNodes(node, obj);
                }
            },
            "Server": function(node, obj) {
                // when having multiple Server types, we prefer WMS
                if ((!obj.service && !obj.version) || 
                    (obj.service != 'urn:ogc:serviceType:WMS')) {
                        obj.service = node.getAttribute("service");
                        obj.version = node.getAttribute("version");
                        this.readChildNodes(node, obj);
                }
            },
            "Name": function(node, obj) {
                obj.name = this.getChildValue(node);
                this.readChildNodes(node, obj);
            },
            "Title": function(node, obj) {
                obj.title = this.getChildValue(node);
                this.readChildNodes(node, obj);
            },
            "StyleList": function(node, obj) {
                obj.options.styles = [];
                this.readChildNodes(node, obj.options.styles);
            },
            "Style": function(node, obj) {
                var style = {};
                obj.push(style);
                this.readChildNodes(node, style);
            },
            "LegendURL": function(node, obj) {
                var legend = {};
                obj.legend = legend;
                this.readChildNodes(node, legend);
            },
            "OnlineResource": function(node, obj) {
                obj.href = this.getAttributeNS(node, this.namespaces.xlink, "href");
                this.readChildNodes(node, obj);
            },
            "Window": function(node, obj) {
              // read width/height attributes
              obj.size = new OpenLayers.Size(node.getAttribute('width'), node.getAttribute('height'));
            },
            "Extension": function(node, obj) {
              // todo: read ol:baselayers, ol:maxExtent
              this.readChildNodes(node, obj);
            }
        },
        "ol": {
          "baselayers": function(node, obj) {
            // add baselayers to context
            obj.baselayers = this.getChildValue(node);
            
            // adjust layer options for spherical mercator projection of baselayers
            if (obj.baselayers.indexOf('google') >= 0 
              || obj.baselayers.indexOf('osm') >= 0
              || obj.baselayers.indexOf('bing') >= 0
              || obj.baselayers.indexOf('ve') >= 0) {
              // set projection and displayprojection
              if (! this.options.layerOptions['OpenLayers.Layer.Vector']) {
                this.options.layerOptions['OpenLayers.Layer.Vector'] = {};
              }
              this.options.layerOptions['OpenLayers.Layer.Vector'].projection = new OpenLayers.Projection('EPSG:4326');
              }
            }
        },
        "ows": {
            "OutputFormat": function(node, obj) {
                obj.format = this.getChildValue(node);
                this.readChildNodes(node, obj);
            },
            "BoundingBox": function(node, obj) {
               //obj.projection = new OpenLayers.Projection(node.getAttribute("crs"));
               obj.projection = node.getAttribute("crs"); //as in WMC context
               this.readChildNodes(node, obj);
            },
            "LowerCorner": function(node, obj) {
                var str = this.getChildValue(node).replace(this.regExes.trimSpace, "");
                str = str.replace(this.regExes.trimComma, ",");
                var pointList = str.split(this.regExes.splitSpace);
                obj.left = pointList[0];
                obj.bottom = pointList[1];
            },
            "UpperCorner": function(node, obj) {
                var str = this.getChildValue(node).replace(this.regExes.trimSpace, "");
                str = str.replace(this.regExes.trimComma, ",");
                var pointList = str.split(this.regExes.splitSpace);
                obj.right = pointList[0];
                obj.top = pointList[1];
                obj.bounds = new OpenLayers.Bounds(obj.left, obj.bottom, obj.right, obj.top);
            },
            "Title": function(node, obj) {
                obj.title = this.getChildValue(node);
            }
        },
        "gml": OpenLayers.Format.GML.v2.prototype.readers.gml,
        "sld": OpenLayers.Format.SLD.v1_0_0.prototype.readers.sld,
        "feature": OpenLayers.Format.GML.v2.prototype.readers.feature
    },

    /**
     * Method: writeLayerScaleInfo
     * Write out the scale information for a Layer tag
     *
     * Parameters:
     * layerOptions - {Object} the options object of a OpenLayers.Layer
     * node - {Object} the DOM node to append to
     */
    writeLayerScaleInfo: function(layerOptions, node) {    
        if (layerOptions.maxScale) {
            this.writeNode("sld:MinScaleDenominator", layerOptions.maxScale, node);
        }
        if (layerOptions.minScale) {
            this.writeNode("sld:MaxScaleDenominator", layerOptions.minScale, node);
        }
    },

    /**
     * Property: writers
     * As a compliment to the readers property, this structure contains public
     *     writing functions grouped by namespace alias and named like the
     *     node names they produce.
     */
    writers: {
        "owc": {
            "OWSContext": function(options) {
                var node = this.createElementNSPlus("OWSContext", {
                    attributes: {
                        version: this.VERSION,
                        id: OpenLayers.Util.createUniqueID("OpenLayers_OWSContext_")
                    } 
                }); 
                this.writeNode("General", options, node);
                this.writeNode("ResourceList", options, node);
                return node; 
            },
            "General": function(options) {
                var node = this.createElementNSPlus("General");
                this.writeNode("ows:BoundingBox", options, node);
                this.writeNode("ows:Title", options.title, node);
                return node;
            },
            "ResourceList": function(options) {
                var node = this.createElementNSPlus("ResourceList");
                this.resourceList = node;
                for (var i=0, len=options.layers.length; i<len; i++) {
                    var layer = options.layers[i];
                    if (layer instanceof OpenLayers.Layer.WMS) {
                        this.writeNode("_WMS", layer, node);
                    } else if (layer instanceof OpenLayers.Layer.Vector) {
                        if (layer.protocol instanceof OpenLayers.Protocol.WFS.v1) {
                            this.writeNode("_WFS", layer, node);
                        } else if (layer.protocol instanceof OpenLayers.Protocol.HTTP) {
                            if (layer.protocol.format instanceof OpenLayers.Format.GML) {
                                layer.protocol.format.version = "2.1.2";
                                this.writeNode("_GML", layer, node);
                            } else if (layer.protocol.format instanceof OpenLayers.Format.KML) {
                                layer.protocol.format.version = "2.2";
                                this.writeNode("_KML", layer, node);
                            }
                        } else {
                            // write out as inline GML since we have no idea
                            // about the original Format
                            this.setNamespace("feature", this.featureNS);
                            this.writeNode("_InlineGeometry", layer, node);
                        }
                    }
                }
                return node;
            },
            "Server": function(options) {
                var node = this.createElementNSPlus("Server", {attributes: {
                    version: options.version,
                    service: options.service }
                });
                this.writeNode("OnlineResource", options, node);
                return node;
            },
            "OnlineResource": function(options) {
                var node = this.createElementNSPlus("OnlineResource", {attributes: {
                    "xlink:href": options.href }
                });
                return node;
            },
            "InlineGeometry": function(layer) {
                var node = this.createElementNSPlus("InlineGeometry");
                this.writeNode("gml:boundedBy", layer.getDataExtent(), node);
                for (var i=0, len=layer.features.length; i<len; i++) {
                    this.writeNode("gml:featureMember", layer.features[i], node);
                }
                return node;
            },
            "StyleList": function(styles) {
                var node = this.createElementNSPlus("StyleList");
                for (var i=0, len=styles.length; i<len; i++) {
                    this.writeNode("Style", styles[i], node);
                }
                return node;
            },
            "Style": function(style) {
                var node = this.createElementNSPlus("Style");
                this.writeNode("Name", style, node);
                this.writeNode("Title", style, node);
                this.writeNode("LegendURL", style, node);
                return node;
            },
            "Name": function(obj) {
                var node = this.createElementNSPlus("Name", {
                    value: obj.name });
                return node;
            },
            "Title": function(obj) {
                var node = this.createElementNSPlus("Title", {
                    value: obj.title });
                return node;
            },
            "LegendURL": function(style) {
                var node = this.createElementNSPlus("LegendURL");
                this.writeNode("OnlineResource", style.legend, node);
                return node;
            },
            "__WMS": function(layer) {
                var node = this.createElementNSPlus("Layer", {attributes: {
                    name: layer.params.LAYERS,
                    queryable: layer.queryable ? "1" : "0",
                    hidden: layer.visibility ? "0" : "1" }
                });
                this.writeNode("ows:Title", layer.name, node);
                this.writeNode("ows:OutputFormat", layer.params.FORMAT, node);
                this.writeNode("Server", {service: 'urn:ogc:serviceType:WMS',
                    version: layer.params.VERSION, href: layer.url}, node);
                if (layer.styles && layer.styles.length > 0) {
                    this.writeNode("StyleList", layer.styles, node);
                }
                this.writeLayerScaleInfo(layer.options, node);
                return node;
            },
            "_WMS": function(layer) {
                if (layer.nestingPath) {
                    // nestingPath is for instance General Bathymetric Chart/GEBCO
                    var title;
                    if (layer.nestingPath.indexOf("/") >= 0) {
                        title = layer.nestingPath.substring(0, layer.nestingPath.indexOf("/"));
                        layer.nestingPath  = layer.nestingPath.substring(layer.nestingPath.indexOf("/")+1, layer.nestingPath.length);
                    } else {
                        title = layer.nestingPath;
                        delete layer.nestingPath;
                    }
                    var node = null;
                    // protect ourselves against MACHU test areas/MACHU test areas
                    if (title !== layer.previousTitle) {
                        // we should be checking if there is already a Layer node with this title
                        var layers = this.getElementsByTagNameNS(
                            this.resourceList, this.namespaces[this.defaultPrefix], "Layer"
                        );
                        for (var i=0,len=layers.length; i<len; i++) {
                            var l = layers[i];
                            var titles = this.getElementsByTagNameNS(
                                l, this.namespaces.ows, "Title");
                            if (titles && titles.length > 0 && titles[0].firstChild.nodeValue === title) {
                                node = l;
                                break;
                            }
                        }
                    }
                    // if not found, create a new one
                    if (node === null) {
                         node = this.createElementNSPlus("Layer");
                         this.writeNode("ows:Title", title, node);
                    }
                    layer.previousTitle = title;
                    this.writeNode("_WMS", layer, node);
                    return node;
                } else { 
                    // write out the actual WMS layer
                    return this.writeNode("__WMS", layer);
                }
            },
            "_WFS": function(layer) {
                var node = this.createElementNSPlus("Layer", {attributes: {
                    name: layer.protocol.featureType,
                    hidden: layer.visibility ? "0" : "1" }
                });
                this.writeNode("ows:Title", layer.name, node);
                this.writeNode("Server", {service: 'urn:ogc:serviceType:WFS', 
                    version: layer.protocol.version, 
                    href: layer.protocol.url}, node);
                this.writeLayerScaleInfo(layer.options, node);
                return node;
            },
            "_InlineGeometry": function(layer) {
                var node = this.createElementNSPlus("Layer", {attributes: {
                    name: this.featureType,
                    hidden: layer.visibility ? "0" : "1" }
                });
                this.writeNode("ows:Title", layer.name, node);
                this.writeNode("InlineGeometry", layer, node);
                return node;
            },
            "_GML": function(layer) {
                var node = this.createElementNSPlus("Layer");
                this.writeNode("ows:Title", layer.name, node);
                this.writeNode("Server", {service: 'urn:ogc:serviceType:GML', 
                    href: layer.protocol.url, version: layer.protocol.format.version}, node);
                this.writeLayerScaleInfo(layer.options, node);
                return node;
            },
            "_KML": function(layer) {
                var node = this.createElementNSPlus("Layer");
                this.writeNode("ows:Title", layer.name, node);
                // TODO: Format.KML should advertize its version
                this.writeNode("Server", {service: 'urn:ogc:serviceType:KML',
                    version: layer.protocol.format.version, href: layer.protocol.url}, node);
                this.writeLayerScaleInfo(layer.options, node);
                return node;
            }
        },
        "ows": {
            "BoundingBox": function(options) {
                var node = this.createElementNSPlus("ows:BoundingBox", {
                    attributes: {
                        crs: options.projection
                    }
                });
                this.writeNode("ows:LowerCorner", options, node);
                this.writeNode("ows:UpperCorner", options, node);
                return node;
            },
            "LowerCorner": function(options) {
                var node = this.createElementNSPlus("ows:LowerCorner", { 
                    value: options.bounds.left + " " + options.bounds.bottom });
                return node;
            },
            "UpperCorner": function(options) {
                var node = this.createElementNSPlus("ows:UpperCorner", {
                    value: options.bounds.right + " " + options.bounds.top });
                return node;
            },
            "Title": function(title) {
                var node = this.createElementNSPlus("ows:Title", {
                    value: title });
                return node;
            },
            "OutputFormat": function(format) {
                var node = this.createElementNSPlus("ows:OutputFormat", {
                    value: format });
                return node;
            }
        },
        "gml": OpenLayers.Util.applyDefaults({
            "boundedBy": function(bounds) {
                var node = this.createElementNSPlus("gml:boundedBy");
                this.writeNode("gml:Box", bounds, node);
                return node;
            }
        }, OpenLayers.Format.GML.v2.prototype.writers.gml),
        "sld": OpenLayers.Format.SLD.v1_0_0.prototype.writers.sld,
        "feature": OpenLayers.Format.GML.v2.prototype.writers.feature
    },
    
    CLASS_NAME: "OpenLayers.Format.OWSContext.v0_3_1" 

});
