/* Copyright (c) 2006-2008 MetaCarta, Inc., published under the Clear BSD
 * license.  See http://svn.openlayers.org/trunk/openlayers/license.txt for the
 * full text of the license. */

/**
 * @requires OpenLayers/Format/XML.js
 * @requires OpenLayers/Feature/Vector.js
 * @requires OpenLayers/Geometry/Point.js
 * @requires OpenLayers/Geometry/LineString.js
 * @requires OpenLayers/Geometry/Polygon.js
 */

if (!$defined(MappingKit.Format)) {
  MappingKit.Format = {};
}

/**
 * Class: OpenLayers.Format.GeoRSS
 * Read/write GeoRSS parser. Create a new instance with the 
 *     <OpenLayers.Format.GeoRSS> constructor.
 *
 * Inherits from:
 *  - <OpenLayers.Format.XML>
 */
MappingKit.Format.MapLayerRSS = OpenLayers.Class(OpenLayers.Format.XML, {
    
    /**
     * APIProperty: rssns
     * {String} RSS namespace to use. Defaults to
     *   "http://backend.userland.com/rss2"
     */
    rssns: "http://backend.userland.com/rss2",
    
    /**
     * APIProperty: featurens
     * {String} Feature Attributes namespace.  Defaults to
     *    "http://mapserver.gis.umn.edu/mapserver"
     */
    featureNS: "http://mapserver.gis.umn.edu/mapserver",
    
    /**
     * APIProperty: georssns
     * {String} GeoRSS namespace to use.  Defaults to
     *     "http://www.georss.org/georss"
     */
    georssns: "http://www.georss.org/georss",

    /**
     * APIProperty: geons
     * {String} W3C Geo namespace to use.  Defaults to
     *     "http://www.w3.org/2003/01/geo/wgs84_pos#"
     */
    geons: "http://www.w3.org/2003/01/geo/wgs84_pos#",
    
    /**
     * APIProperty: featureTitle
     * {String} Default title for features.  Defaults to "Untitled"
     */
    featureTitle: "Untitled",
    
    /**
     * APIProperty: featureDescription
     * {String} Default description for features.  Defaults to "No Description"
     */
    featureDescription: "No Description",
    
    /**
     * Property: gmlParse
     * {Object} GML Format object for parsing features
     * Non-API and only created if necessary
     */
    gmlParser: null,

    /**
     * APIProperty: xy
     * {Boolean} Order of the GML coordinate: true:(x,y) or false:(y,x)
     * For GeoRSS the default is (y,x), therefore: false
     */ 
    xy: false,
    
    /**
     * Property: layers
     * {Array(<OpenLayers.Layer.WMS>)}
     */
    layers: null,
    
    /**
     * Property: location
     * {String} location/url of taxonomy feed with Map Layers
     */
    location: null,
    
    /**
     * Property: layerOptions
     * {Object} Default options for layers created by the parser. These
     *     options are overridden by the options which are read from the 
     *     capabilities document.
     */
    layerOptions: null,
    
    /**
     * Constructor: OpenLayers.Format.GeoRSS
     * Create a new parser for GeoRSS.
     *
     * Parameters:
     * options - {Object} An optional object whose properties will be set on
     *     this instance.
     */
    initialize: function(options) {
        OpenLayers.Format.XML.prototype.initialize.apply(this, [options]);
        
        this.layers = [];
    },
    
    destroy: function() {
        OpenLayers.Format.XML.prototype.destroy.apply(this, arguments);
        
        this.layers = null;
    },
    
    /**
     * Method: getChildValue
     *
     * Parameters:
     * node - {DOMElement}
     * nsuri - {String} Child node namespace uri ("*" for any).
     * name - {String} Child node name.
     * def - {String} Optional string default to return if no child found.
     *
     * Returns:
     * {String} The value of the first child with the given tag name.  Returns
     *     default value or empty string if none found.
     */
    getChildValue: function(node, nsuri, name, def) {
        var value;
        try {
            value = this.getElementsByTagNameNS(node, nsuri, name)[0].firstChild.nodeValue;
        } catch(e) {
            value = (def == undefined) ? "" : def;
        }
        return value;
    },
    
    /**
     * Method: createLayerFromItem
     * Return a feature from a GeoRSS Item.
     *
     * Parameters:
     * item - {DOMElement} A GeoRSS item node.
     *
     * Returns:
     * {<OpenLayers.Layer>} A layer representing the item.
     */
    createLayerFromItem: function(item) {
        /* Provide defaults for title and description */
        var title = this.getChildValue(item, "*", "title", this.featureTitle);
        
        // wms parameters
        var serviceurl = this.getChildValue(item, '*', 'serviceurl');
        var layers = this.getChildValue(item, '*', 'layers');
        var legendurl = this.getChildValue(item, '*', 'legendurl');
        var info_format = this.getChildValue(item, '*', 'info_format');
        var maxextent = this.getChildValue(item, '*', 'maxextent');
        var queryable = this.getChildValue(item, '*', 'queryable') == 'true';
        
        var layer_type = this.getChildValue(item, '*', 'layer_type');
        
        // create layer (title, url, request options, layer options
        var requestOptions = {'layers': layers
          , 'format': 'image/png'
          , 'transparent': true
        };
        /*
        var pos = serviceurl.indexOf('?');
        if (pos > 0) {
          var query_params = serviceurl.substr(pos + 1).split('&');
          alert(query_params);
        }
        */
        
        var layerOptions = { opacity: 1.0
          , isBaseLayer: false
          , displayInLegend: true
          , singleTile: serviceurl.indexOf('tilecache') == -1 // single tile when no tilecache
          , queryable: queryable
        };
        layerOptions.maxExtent = new OpenLayers.Bounds.fromString(maxextent);
        if (legendurl != '') {
          layerOptions.legend = {};
          layerOptions.legend.href = legendurl;
        }
        if (info_format) {
          layerOptions.info_formats = info_format.split(',');
        }
        
        var layer = null;
        switch (layer_type) {
        case 'tilecache':
          break;
        case 'tms':
          break;
        case 'wfs':
          break;
        default:
        case 'wms':
          var layer = new OpenLayers.Layer.WMS(title, serviceurl, requestOptions, layerOptions);
          break;
        }
        return layer;
    },
    
    /**
     * APIMethod: read
     * Return a list of features from a GeoRSS doc
     
     * Parameters:
     * data - {Element} 
     *
     * Returns:
     * An Array of <OpenLayers.Feature.Vector>s
     */
    read: function(doc) {
        if (typeof doc == "string") { 
            doc = OpenLayers.Format.XML.prototype.read.apply(this, [doc]);
        }

        /* Try RSS items first, then Atom entries */
        var itemlist = null;
        itemlist = this.getElementsByTagNameNS(doc, '*', 'item');
        if (itemlist.length == 0) {
            itemlist = this.getElementsByTagNameNS(doc, '*', 'entry');
        }
        
        var numItems = itemlist.length;
        var features = new Array(numItems);
        for(var i=0; i<numItems; i++) {
            features[i] = this.createLayerFromItem(itemlist[i]);
        }
        return features;
    },
    
    CLASS_NAME: "MappingKit.Format.MapLayerRSS"
});

