<?php

/**
 * @file
 * Parser for WMS Capabilities.
 * May correct the capabilities in the following ways:
 * - convert srs to uppercase
 * - check if href is valid and not localhost and overwrite if not
 */

// include xml parser
module_load_include('inc', 'mappingkit', 'includes/xml.parser');

class WMS_Parser extends XML_Parser {
  
  public $layers;
  
  /**
   * OnlineResource in Service section.
   */
  private $service_href;
  private $force_href;
  
  /**
   * List of error messages.
   */
  public $errors;
  
  function __construct() {
    parent::__construct();
    
    $this->namespaces['sld'] = "http://www.opengis.net/sld";
    
    $this->defaultNamespacePrefix = 'wms';
    $this->layers = array();
    $this->errors = array();
  }
  
  /**
   * Force service url to a given one. The url should end with ? or &.
   */
  function force_service_url($url) {
    $this->force_href = $url . (strpos($url, '?') === FALSE ? '?' : '&');
  }
  
  function load($xml) {
    parent::load($xml);
    
    // check for tiled layers
    if (isset($this->content['capability']['extension']['tileset'])) {
      foreach ($this->content['capability']['extension']['tileset'] as $tileset) {
        $tiled_layers = explode(',', $tileset['Layers']);
        foreach ($tiled_layers as $name) {
          foreach ($this->layers as &$layer) {
            if (isset($layer['Name']) && $layer['Name'] == $name) {
              $layer['tileset'] = $tileset;
              break;
            }
          }
        }
      }
    }
    
    //drupal_set_message('layers '.var_export($this->layers, TRUE));
    
    return $this->content;
  }
  
  function read_wms_Service(&$context, $node) {
    $this->run_child_nodes($context, $node);
    
    $this->service_href = $context['href'];
  }
  
  function read_wms_Name(&$context, $node) {
    $this->read_property('Name', $context, $node);
  }
  
  function read_wms_Title(&$context, $node) {
    $this->read_property('Title', $context, $node);
  }
  
  function read_wms_Abstract(&$context, $node) {
    $this->read_property('Abstract', $context, $node);
  }
  
  function read_wms_Fees(&$context, $node) {
    $this->read_property('Fees', $context, $node);
  }
  
  function read_wms_AccessConstraints(&$context, $node) {
    $this->read_property('AccessConstraints', $context, $node);
  }
  
  function read_wms_KeywordList(&$context, $node) {
    $context['Keyword'] = array();
    $this->run_child_nodes($context, $node);
  }
  
  function read_wms_Keyword(&$context, $node) {
    // note: can be multiple formats
    if (!isset($context['Keyword'])) {
      $context['Keyword'] = array();
    }
    $tmp = array();
    $this->read_property('Keyword', $tmp, $node);
    $context['Keyword'][] = $tmp['Keyword'];
  }
  
  function read_wms_OnlineResource(&$obj, $node) {
    global $_mappingkit_logger;
    
    $obj['href'] = $node->getAttributeNS('http://www.w3.org/1999/xlink', 'href');
    
    if (!valid_url($obj['href'], TRUE) || parse_url($obj['href'], PHP_URL_HOST) == 'localhost') {
      $this->errors[] = t('Invalid href: ', array('%url' => $obj['href']));
      if (isset($this->service_href) && isset($this->force_href)) {
        $_mappingkit_logger->debug("invalid href {$obj['href']} replaced with {$this->force_href}");
        $obj['href'] = str_replace($this->service_href, $this->force_href, $obj['href']);
      }
    }
  }

  function read_wms_ContactInformation(&$context, $node) {
    $context['ContactInformation'] = array();
    $this->run_child_nodes($context['ContactInformation'], $node);
  }
  
  function read_wms_ContactPersonPrimary(&$context, $node) {
    $context['ContactPersonPrimary'] = array();
    $this->run_child_nodes($context['ContactPersonPrimary'], $node);
  }
  
  function read_wms_ContactPerson(&$context, $node) {
    $this->read_property('ContactPerson', $context, $node);
  }
  
  function read_wms_ContactOrganization(&$context, $node) {
    $this->read_property('ContactOrganization', $context, $node);
  }
  
  function read_wms_ContactElectronicMailAddress(&$context, $node) {
    $this->read_property('ContactElectronicMailAddress', $context, $node);
  }
  
  function read_wms_Capability(&$context, $node) {
    $capability = array();
    $this->run_child_nodes($capability, $node);
    $context['capability'] = $capability;
  }
  
  function read_wms_Request(&$context, $node) {
    $request = array();
    $this->run_child_nodes($request, $node);
    $context['request'] = $request;
  }
  
  function read_wms_GetMap(&$context, $node) {
    $request = array();
    $this->run_child_nodes($request, $node);
    $context['GetMap'] = $request;
  }
  
  // 1.0.7
  function read_wms_FeatureInfo(&$context, $node) {
    $request = array();
    $this->run_child_nodes($request, $node);
    $context['GetFeatureInfo'] = $request;
  }
  
  // 1.1.1
  function read_wms_GetFeatureInfo(&$context, $node) {
    $request = array();
    $this->run_child_nodes($request, $node);
    $context['GetFeatureInfo'] = $request;
  }
  
  function read_wms_DCPType(&$context, $node) {
    $this->run_child_nodes($context, $node);
  }
  
  function read_wms_HTTP(&$context, $node) {
    $this->run_child_nodes($context, $node);
  }
  
  function read_wms_Get(&$context, $node) {
    $context['Get'] = array();
    $this->run_child_nodes($context['Get'], $node);
  }
  
  function read_wms_GetCapabilities(&$context, $node) {
    $request = array();
    $this->run_child_nodes($request, $node);
    $context['GetCapabilities'] = $request;
  }
  
  function read_wms_Format(&$context, $node) {
    // note: can be multiple formats
    if (!isset($context['Format'])) {
      $context['Format'] = array();
    }
    $tmp = array();
    $this->read_property('Format', $tmp, $node);
    $context['Format'][] = $tmp['Format'];
  }
  
  function read_wms_SRS(&$context, $node) {
    if (!isset($context['SRS'])) {
      $context['SRS'] = array();
    }
    $tmp = array();
    $this->read_property('SRS', $tmp, $node);
    // todo? old wms format contains multiple srs separated by spaces
    $context['SRS'][] = strtoupper($tmp['SRS']);
  }
  
  function read_wms_BoundingBox(&$context, $node) {
    // todo: multiple BoundingBox
    if (!isset($context['BoundingBox'])) {
      $context['BoundingBox'] = array();
    }
    $srs = strtoupper($node->getAttribute('SRS'));
    $context['BoundingBox'][$srs] = array(
      'SRS' => $srs
      ,'minx' => $node->getAttribute('minx')
      , 'miny' => $node->getAttribute('miny')
      , 'maxx' => $node->getAttribute('maxx')
      , 'maxy' => $node->getAttribute('maxy')
      );
  }
  
  function read_wms_LatLonBoundingBox(&$context, $node) {
    // todo: multiple BoundingBox
    if (!isset($context['BoundingBox'])) {
      $context['BoundingBox'] = array();
    }
    $srs = 'EPSG:4326';
    $context['BoundingBox'][$srs] = array(
      'SRS' => $srs
      , 'minx' => $node->getAttribute('minx')
      , 'miny' => $node->getAttribute('miny')
      , 'maxx' => $node->getAttribute('maxx')
      , 'maxy' => $node->getAttribute('maxy')
      );
  }
  
  function read_wms_Style(&$context, $node) {
    // multiple styles
    if (!isset($context['styles'])) {
      $context['styles'] = array();
    }
    $style = array();
    $this->run_child_nodes($style, $node);
    $context['styles'][] = $style;
  }
  
  /* */
  function read_wms_LegendURL(&$style, $node) {
    $legend = array();
    if ($node->hasAttribute('width')) {
      $legend['width'] = $node->getAttribute('width');
    }
    if ($node->hasAttribute('height')) {
      $legend['height'] = $node->getAttribute('height');
    }
    $links = $node->getElementsByTagName('OnlineResource');
    if ($links->length > 0) {
      $this->read_wms_OnlineResource($legend, $links->item(0));
    }
    $style['legend'] = $legend;
  }/* */
  
  function read_wms_Width(&$context, $node) {
    $this->read_property('Width', $context, $node);
  }
  
  function read_wms_Height(&$context, $node) {
    $this->read_property('Height', $context, $node);
  }
  
  function read_wms_Resolutions(&$context, $node) {
    $this->read_property('Resolutions', $context, $node);
  }
  
  function read_wms_Exception(&$context, $node) {
    $context['Exception'] = array();
    $this->run_child_nodes($context['Exception'], $node);
  }
  
  function read_wms_VendorSpecificCapabilities(&$context, $node) {
    $extension = array();
    $this->run_child_nodes($extension, $node);
    $context['extension'] = $extension;
  }
  
  function read_wms_TileSet(&$context, $node) {
    // multiple tilesets
    if (!isset($context['tileset'])) {
      $context['tileset'] = array();
    }
    $tileset = array();
    $this->run_child_nodes($tileset, $node);
    $context['tileset'][] = $tileset;
  }
  
  /**
   * Read Layers for TileSet.
   */
  function read_wms_Layers(&$context, $node) {
    $this->read_property('Layers', $context, $node);
  }
  
  /**
   * Read Layer definitions for capabilities.
   */
  function read_wms_Layer(&$context, $node) {
    if (!isset($context['layers'])) {
      $context['layers'] = array();
    }
    
    $layer = array('BoundingBox' => array());
    $attribs = array('cascaded', 'opaque', 'queryable');
    if (isset($context['request'])) {
      // top layer
      foreach ($attribs as $attrib) {
        if ($node->hasAttribute($attrib)) {
          $layer[$attrib] = $node->getAttribute($attrib);
        }
        else {
          // defaults to 0
          $layer[$attrib] = 0;
        }
      }
    }
    else {
      // inherit: queryable, cascaded, opaque, noSubsets, fixedWidth, fixedHeight (default to 0)
      foreach ($attribs as $attrib) {
        if (isset($context[$attrib])) {
          // inherit
          $layer[$attrib] = $context[$attrib];
        }
        if ($node->hasAttribute($attrib)) {
          // override
          $layer[$attrib] = $node->getAttribute($attrib);
        }
      }
      
      // inherits parent: srs, styles, BoundingBox
      if (isset($context['SRS'])) {
        $layer['SRS'] = $context['SRS'];
      }
      if (isset($context['Style'])) {
        $layer['Style'] = $context['Style'];
      }
      if (isset($context['BoundingBox'])) {
        $layer['BoundingBox'] = $context['BoundingBox'];
      }
    }
    
    // assign first to arrays before doing run_child_nodes in order to get
    // the layers in order of appearance
    $context['layers'][] = &$layer;
    $this->layers[] = &$layer;
    
    $this->run_child_nodes($layer, $node);
  }
  
  // ScaleHint min max
}

